<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use App\Http\Requests\API\File\DownloadRequest;
use App\Http\Requests\API\File\FileUploadRequest;
use App\Http\Requests\API\File\UploadRequest;
use App\Http\Resources\API\FileUploadSuccessResource;
use App\Models\Enums\FileObjectType;
use App\Models\File;
use Carbon\Carbon;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;

class FileController extends Controller
{

    public function upload(UploadRequest $request)
    {

        $file = $request->file('file');
        $ext = $request->file('file')->getClientOriginalExtension() ? '.'.$request->file('file')->getClientOriginalExtension() : null;
        $fileName = $request->fileName ? $request->fileName : Carbon::now()->timestamp. '_' . Str::random(10).$ext;
        $path =  'uploads/' .date('Ymd/').$fileName;
        Storage::put($path, file_get_contents($file->getRealPath()));
        return [
        'fileName' => $fileName,
        'url' => $path,
        'fullUrl' => Storage::url($path),
        'ext' => $ext,
    ];
    }


    public function download(DownloadRequest $request)
    {
        $url=$request->url;
        return Storage::download($url);
    }

    /**
     * 文件上传
     *
     * @param FileUploadRequest $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function uploads(FileUploadRequest $request)
    {
        $names = $request->get("file_names", []);
        $fileObjectType = FileObjectType::from($request->object_type);

        foreach ($request->file("files") as $file) {
            if (! $file->isValid()) {
                return $this->badRequest("File upload failed.");
            }
        }

        $items = [];
        foreach ($request->file("files") as $index => $file) {
            $pathname = $file->storeAs(
                sprintf("c%s/%s/%s", Auth::user()->company_id, $fileObjectType->value, date("Ymd")),
                sprintf("%s.%s", md5(uniqid()), $file->extension()),
            );

            if (! $pathname) {
                return $this->badRequest("File upload failed.");
            }

            $items[] = [
                'pathname' => $pathname,
                'title' => $names[$index] ?? $file->getClientOriginalName(),
                'size' => $file->getSize(),
                'extension' => $file->extension(),
                'object_type' => $request->object_type,
                'object_id' => $request->object_id,
                'created_by' => Auth::id(),
                'company_id' => Auth::user()->company_id,
            ];
        }

        $uploadedFiles = [];
        foreach ($items as $item) {
            $file = File::query()->create($item);

            $uploadedFiles[] = new FileUploadSuccessResource($file);
        }

        return $this->success([
            'data' => $uploadedFiles
        ]);
    }
}
